
// =======================================================================================
//  CFilter_MultiPole_FixedSampling
// =======================================================================================
//  This is an IIR filter, like multiple RC cells
//  The sampling frequency is fixed, so tau is precalculated for the maximum efficiency
//  The tau is correct so the fCut is alwaiys at -3dB point, also using many poles
// =======================================================================================

class CFilter_MultiPole_FixedSampling
{
protected:
    static const int MAX_POLES = 100;

    double sampleFreq;
	double fCut3dB;
	int nPoles;
	bool highPass;

	double tau;
	double fC;
	double sampleTime;
	double tauSamples;
	double ampFactor;

	double buffer[MAX_POLES + 1];
	double bufferHiPass[MAX_POLES + 1];

	void CalculateAmpFactor()
    {
		if (fCut3dB == 0 || fCut3dB > sampleFreq / 2)
		{
			nPoles = 0;
			return;
		}
		// -----------------------------------------------------------------
		if (highPass) fC = fCut3dB * sqrt(pow(2, (1.0 / nPoles)) -1);
		else fC = fCut3dB / sqrt(pow(2, (1.0 / nPoles)) -1); 
		tau = 1 / (2 * PI * fC);
		sampleTime = 1 / sampleFreq;
		tauSamples = tau / sampleTime;
		ampFactor = exp(-1.0 / tauSamples);
    }

	void SetDefaultValues()
    {
		sampleFreq = 192000;
        fCut3dB = 100;
		highPass = false;
        nPoles = 1;
		CalculateAmpFactor();
    }

public:

	CFilter_MultiPole_FixedSampling()
    {
		SetDefaultValues();
    }

	~CFilter_MultiPole_FixedSampling()
	{
	}

	void SetHiPass(bool _hiPass)
    {
		if (highPass == _hiPass) return;
		highPass = _hiPass;
		CalculateAmpFactor();
    }

    void SetFrequency(double _fCut3dB)
    {
		if (fCut3dB == _fCut3dB) return;
        fCut3dB = _fCut3dB;
		CalculateAmpFactor();
    }

	double GetFrequency()
    {
		return fCut3dB;
    }

	void SetSampleFrequency(double _sampleFreq)
    {
		if (sampleFreq == _sampleFreq) return;
		if (_sampleFreq < 0) _sampleFreq = 0;
        sampleFreq = _sampleFreq;
		CalculateAmpFactor();
    }

    void SetNpoles(int _nPoles)
    {
		if (_nPoles > MAX_POLES) _nPoles = MAX_POLES;
		if (nPoles == _nPoles) return;
        nPoles = _nPoles;
		CalculateAmpFactor();
    }
    
	double	Exec(double inValue)
	{
		if (nPoles == 0) return inValue;
		
		if (highPass)
		{
			bufferHiPass[nPoles] = inValue;
			for (int i = nPoles - 1; i >= 0; i--)
			{
				buffer[i] = (1.0 - ampFactor) * bufferHiPass[i + 1] + ampFactor * buffer[i];
				bufferHiPass[i] = bufferHiPass[i + 1] - buffer[i];
			}
		}
		else
		{
			buffer[nPoles] = inValue;
			for (int i = nPoles - 1; i >= 0; i--)
			{
				buffer[i] = (1.0 - ampFactor) * buffer[i + 1] + ampFactor * buffer[i];
			}
		}
		if (highPass) return bufferHiPass[0];
		else return buffer[0];
	}

};